@@ -66,7 +66,7 @@ def test_build_with_scheme_and_host():
             id="port-only",
         ),
         pytest.param(
-            "", TypeError, r"^The port is required to be int\.$", id="port-str"
+            "", TypeError, r"^The port is required to be int, got .*\.$", id="port-str"
         ),
     ],
 )
@@ -140,12 +140,18 @@ def test_build_with_invalid_host(host: str, is_authority: bool):
 
 
 def test_build_with_authority():
-    url = URL.build(scheme="http", authority="степан:bar@host.com:8000", path="path")
+    url = URL.build(scheme="http", authority="степан:bar@host.com:8000", path="/path")
     assert (
         str(url) == "http://%D1%81%D1%82%D0%B5%D0%BF%D0%B0%D0%BD:bar@host.com:8000/path"
     )
 
 
+def test_build_with_authority_no_leading_flash():
+    msg = r"Path in a URL with authority should start with a slash \('/'\) if set"
+    with pytest.raises(ValueError, match=msg):
+        URL.build(scheme="http", authority="степан:bar@host.com:8000", path="path")
+
+
 def test_build_with_authority_without_encoding():
     url = URL.build(
         scheme="http", authority="foo:bar@host.com:8000", path="path", encoded=True
@@ -262,6 +268,33 @@ def test_build_already_encoded():
     assert str(u) == "http://оун-упа.укр/шлях/криївка?ключ=знач#фраг"
 
 
+def test_build_already_encoded_username_password():
+    u = URL.build(
+        scheme="http",
+        host="x.org",
+        path="/x/y/z",
+        query_string="x=z",
+        fragment="any",
+        user="u",
+        password="p",
+        encoded=True,
+    )
+    assert str(u) == "http://u:p@x.org/x/y/z?x=z#any"
+    assert u.host_subcomponent == "x.org"
+
+
+def test_build_already_encoded_empty_host():
+    u = URL.build(
+        host="",
+        path="/x/y/z",
+        query_string="x=z",
+        fragment="any",
+        encoded=True,
+    )
+    assert str(u) == "/x/y/z?x=z#any"
+    assert u.host_subcomponent is None
+
+
 def test_build_percent_encoded():
     u = URL.build(
         scheme="http",
